﻿using System;
//using System.Collections.Generic;
//using System.Linq;
using System.Text;
using System.Xml;
using System.Net;
using System.IO;

namespace Loadcell
{
    //=================================================
    //원격DB 클래스
    public class xWebDB
    {

        String url;
        XmlDocument xdoc = null;
        XmlNodeList rows = null;

        public xWebDB(String url)
        {
            this.url = url.Trim();
            xdoc = new XmlDocument();

        }//생성자

        //--------------------------------------------------------
        private String makeXMLString(String sqlStr)
        {
            String trimedSqlStr = sqlStr.Trim();

            int index = trimedSqlStr.IndexOf(' ');

            String action = trimedSqlStr.Substring(0, index);

            return "<sql action='" + action + "'><![CDATA[" + trimedSqlStr.Substring(index) + "]]></sql>";
        }

        //--------------------------------------------------------
        public string query(String sqlStr)
        {
            try
            {

                HttpWebResponse wRes = null;
                HttpWebRequest wReq = (HttpWebRequest)WebRequest.Create(new Uri(url));
                wReq.Method = "POST"; // 전송 방법 "GET" or "POST"

                byte[] byteArray = Encoding.UTF8.GetBytes(makeXMLString(sqlStr));

                //Console.WriteLine(byteArray.ToString());

                Stream dataStream = wReq.GetRequestStream();
                dataStream.Write(byteArray, 0, byteArray.Length);
                dataStream.Close();

                using (wRes = (HttpWebResponse)wReq.GetResponse())
                {
                    Stream respPostStream = wRes.GetResponseStream();
                    StreamReader readerPost = new StreamReader(respPostStream, Encoding.GetEncoding("utf-8"), true);

                    try
                    {
                        xdoc.LoadXml(readerPost.ReadToEnd());
                        if (xdoc.DocumentElement.GetAttribute("code").Equals("ERROR"))
                        {
                            return ("Query Error:\n" + xdoc.DocumentElement.ToString());
                        }

                        rows = xdoc.DocumentElement.GetElementsByTagName("ROW");

                    }
                    catch (Exception ex)
                    {
                        return ("XML Load Error:\n" + ex.Message);
                    }

                }//using
            }
            catch (WebException ex)
            {
                if (ex.Status == WebExceptionStatus.ProtocolError && ex.Response != null)
                {

                    var resp = (HttpWebResponse)ex.Response;

                    if (resp.StatusCode == HttpStatusCode.NotFound)
                    {
                        return ("query: URL NotFound Error\n" + ex.Message);
                    }
                    else
                    {
                        return ("query: Web Protocol Error: 기타 Error\n" + ex.Message);
                    }
                }
                else
                {
                    return ("query: Web Non Protocol Error\n" + ex.Message);
                }
            }//catch
            catch (Exception ex)
            {
                return ("query: Exception Error\n" + ex.Message);
            }

            return null;

        }//query;

        //-------------------------------
        public int readCount
        {
            get
            {
                return rows.Count;
            }

        }

        //---------------------------------------------
        public string getValue(string elemName, int indx = 0)
        {

            try
            {
                return rows[indx].SelectSingleNode(elemName).FirstChild.Value;
            }
            catch (Exception ex)
            {
                //Console.WriteLine("getValue exception: {0}, index: {1}\n{2}", elemName, indx, rows[indx].InnerXml);
                return "null";
            }
        }

        //----------------------------------------------
        public int getColumnCount()
        {
            if (rows.Count == 0) return 0;
            return rows[0].ChildNodes.Count;
        }

        //----------------------------------------------
        public string getColumnName(int indx)
        {

            if (rows.Count == 0) return null;
            if (indx < 0) return null;
            if (indx >= rows[0].ChildNodes.Count) return null;

            return rows[0].ChildNodes[indx].Name;
        }

        //----------------------------------------------
        public string getXML()
        {
            return xdoc.OuterXml;

        }

        //----------------------------------------------
        public void saveXML()
        {
            xdoc.Save("query.xml");
        }


    }//class webDB
}
